; macro for diagnose LED handling
dialed  macro   mode            ; HB
	b\mode  #14,x:PBD
	endm

; macro for command LED handling
cmdled  macro   mode            ; STA
	b\mode  #13,x:PBD
	endm

; macro for ptt handling
ptt     macro   mode            ; PTT
	b\mode  #0,x:PBD
	endm

; macro for immediate move
movi    macro   data,dest
	move    data,a1
	move    a1,dest
	endm

; macro for entering interrupt service routine
; stores x0,x1 and a,b registers
enter   macro   contex
	move    x,l:<contex+0
	move    a10,l:<contex+1
	move    a2,y:<contex+2
	move    b10,l:<contex+3
	move    b2,y:<contex+4
	endm

; macro for leaving interrupt service routine
; restores x0,x1 and a,b registers
leave   macro   contex
	move    l:<contex+0,x
	move    l:<contex+1,a10
	move    y:<contex+2,a2
	move    l:<contex+3,b10
	move    y:<contex+4,b2
	rti
	endm

; macro for fir filters, Input x0, Output in a
fir     macro   taps,states,coefs
	jmp     _fstart
_sstore dc      0
_cstore dc      0
_fstart move    #taps-1,m0                      ;mod(ntaps)
	move    #taps-1,m4                      ;mod(ntaps)
	movem   p:_sstore,a
	tst     a
	jne     _dof
	move    #states,r0
	movem   r0,p:_sstore
	move    #coefs,r4
	movem   r4,p:_cstore
_dof    movem   p:_sstore,r0                    ;point to filter states
	move    p:_cstore,r4                    ;point to filter coefficients
	nop
	clr     a    x0,x:(r0)+  y:(r4)+,y0     ;save first state
	rep     #taps-1
	mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
	macr    x0,y0,a  (r0)-
	movem   r0,p:_sstore
	movem   r4,p:_cstore
	endm


; macro for Hilbert transformators, Input x0, i-Output in b, q-Output in a
hilbert macro   taps,states,coefs
	jmp     _hstart
_sstore dc      0
_cstore dc      0
_hstart move    #taps-1,m0                      ;mod(ntaps)
	move    #taps-1,m4                      ;mod(ntaps)
	movem   p:_sstore,a
	tst     a
	jne     _doh
	move    #states,r0
	movem   r0,p:_sstore
	move    #coefs,r4
	movem   r4,p:_cstore
_doh    movem   p:_sstore,r0                    ;point to filter states
	move    p:_cstore,r4                    ;point to filter coefficients
	nop
	clr     a    x0,x:(r0)+  y:(r4)+,y0     ;save first state
	rep     #((taps-1)/2)                   ;do first half of filtering
	mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
	move    x0,b                            ;store delayed input in b
	rep     #((taps-1)/2)                   ;do second half of filtering
	mac     x0,y0,a  x:(r0)+,x0  y:(r4)+,y0
	macr    x0,y0,a  (r0)-
	movem   r0,p:_sstore
	movem   r4,p:_cstore
	endm

; macro for generating a precision sine wave
; theta must be declared long and step as two words in y memory
presin  macro   step,theta,where
	move    #$ffff,m0               ;init pointer registers
	move    #$100,r0
	clr     a
	clr     b
	move    l:theta,a10             ;move current phase long to a
	move    y:step+1,b0             ;move low word step to b0
	asl     b                       ;get rid of it's sign bit
	move    y:step,b1               ;move hi word step to b1
	add     b,a                     ;compute next phase
	move    #>$FF,x1
	and     x1,a1                   ;limit phase to 360 degrees
	move    a10,l:theta             ;store new phase value
	move    a1,n0
	lsl     a
	asr     a
	move    a0,y0
	nop
	move    y:(r0+n0),b             ;get first result for n0
	move    b,y:where               ;intermediate store
	move    #>1,x0
	add     x0,a
	and     x1,a1
	move    a1,n0
	nop
	move    y:(r0+n0),b             ;get second result for n0+1
	move    y:where,x1              ;now compute a better value
	sub     x1,b
	move    b,x1
	macr    x1,y0,b
	move    y:where,a
	add     b,a
	move    a,y:where               ;and store the end result
	endm

; macro for generating a precision cos wave
; theta must be declared long and step as two words in y memory
; may only be used in conjunction (immediatly after) precsin
precos  macro   step,theta,where
	move    #$ffff,m0               ;init pointer registers
	move    #$100,r0
	clr     a
	clr     b
	move    l:theta,a10
	move    #>$40,b
	add     b,a
	move    #>$FF,x1
	and     x1,a1
	move    a1,b1
	move    a0,b0
	move    a1,n0
	lsl     a
	asr     a
	move    a0,y0
	move    y:(r0+n0),b
	move    b,y:where
	move    #>1,x0
	add     x0,a
	and     x1,a1
	move    a1,n0
	nop
	move    y:(r0+n0),b
	move    y:where,x1
	sub     x1,b
	move    b,x1
	macr    x1,y0,b
	move    y:where,a
	add     b,a
	move    a,y:where
	endm

; macro for calculating the square of a value
; input in reg output in reg
sqr     macro  reg
	move   reg,x0
	mpy    x0,x0,reg
	endm

; macro for calculating the square root
; Input in a, Output in a
sqrt    macro
	move   a1,y1
	move   a0,y0
	clr    b                          ; init root and guess
	move   #$400000,x1                ; init bit to test
	move   #$400000,x0
	do     #23,_endrt
	mpy    -x0,x0,a                   ; square and negate the guess
	add    y,a                        ; compare to double precision input
	tge    x0,b                       ; update root if input >= guess
	tfr    x1,a                       ; get bit to test
	asr    a                          ; shift to next bit to test
	add    b,a  a,x1                  ; form new guess
	move   a,x0                       ; save new guess
_endrt  move   b,a
	endm

; macro to compute 2^(-16)/a, works only with positive inputs
; input in a output in a
invert  macro
	move  a,x0
	and   #$fe,CCR
	move  #>$0000FF,a
	rep   #24
	div   x0,a
	add   x0,a
	move  a0,x0
	move  x0,a
	endm

;**********************************************************************
